/* 
#---------------------------------------------------------------------+
#  Copyright (c) 2000 NRG Gladstone Operating Services.               |
#               All Rights Reserved.                                  |
#                                                                     |
#                                                                     |
#  This is free software; you can redistribute it and/or modify it    |
#  under the terms of the GNU General Public License as published by  |
#  the Free Software Foundation; either version 2 of the License, or  |
#  (at your option) any later version.                                |
#                                                                     |
#  This software is distributed in the hope that it will be useful,   |
#  but WITHOUT ANY WARRANTY; without even the implied warranty of     |
#  merchantability or fitness for a particular purpose. See the GNU   |
#  General Public License for more details.                           |
#                                                                     |
#  To obtain the GNU General Public License, write to:                |
#                                                                     |
#      Free Software Foundation, Inc.                                 |
#      59 Temple Place - Suite 330                                    |
#      Boston, MA  02111-1307                                         |
#      USA                                                            |
#---------------------------------------------------------------------+
*/

/*
# $Log: om_calls.c,v $
# Revision 1.1  2000/03/19 22:09:05  dbond
# Initial revision
#
#
*/


#include <stdio.h>
#include <stdlib.h>
#include <fox/om_user.h>
#include <fox/om_ecode.h>
#include <string.h>
#include "omrd.h"
#include "ommem.h"

union omdata {
  float Float;
  char Character;
  int Integer;
  char Om_string[33];
  unsigned char Om_bool;
  long Om_lng_int;
  long Om_s_pkbol;
  long Om_l_pkbol;
}; 
extern int maxshm;
char omcalls_v[] = "$Id: om_calls.c,v 1.1 2000/03/19 22:09:05 dbond Exp $";

/**********************************************************************
         function to get data from Foxboro variable
      Uses Fox API getval function Manual B0193BC
      function omget is passwd a namenode struct with the C:B.P
      required in the struct. The value is placed in the struct
      value is returned as a string.
**********************************************************************/
int omget(namenode *node){
  union omdata myval;
  int size,err;
  char mystr[33]="";
  unsigned int status;
  PSAP_ADDR *psap;
  psap = get_cbp(node->name);

  size = sizeof(myval);
  err=om_getval(node->name,VARIABLE,0,(int *)&myval,(int *)&status,&size,psap);
  if(err != OM_SUCCESS) {
  /* return Foxboro error code refer /usr/include/fox/om_ecode.h */
    sprintf(mystr,"ERROR %d",err);
    node->value = strdup(mystr);
    return(err);
  } 
  status &= 0x1f;
  /* decode datatype returned from Fox system and fill in structure */
  switch(status){
  case FLOAT:
    sprintf(mystr,"%10.3f",myval.Float);
    break;
  case INTEGER:
    sprintf(mystr,"%d",myval.Integer);
    break;
  case STRING:
    sprintf(mystr,"%s",myval.Om_string);
    break;
  case OM_BOOL:
    sprintf(mystr,"%u",(unsigned char)myval.Om_bool);
    break;
  case OM_LNG_INT:
    sprintf(mystr,"%ld",myval.Om_lng_int);
    break;
  case OM_S_PKBOL:
    sprintf(mystr,"0x%x",myval.Om_s_pkbol);
    break;
  case OM_L_PKBOL:
    sprintf(mystr,"0x%x",myval.Om_l_pkbol);
    break;
  default:
    sprintf(mystr,"Error 0x%x",status);
    break;
  }
  node->value = strdup(mystr);
  return(err);
}

/**********************************************************************
         function to change data in a  Foxboro variable
      Uses Fox API set_confirm function Manual B0193BC
      The function is supplied the C:B.P in node and the datatype
**********************************************************************/



int omset(namenode *node,unsigned int  type){
  union omdata myval;
  unsigned int status;
  int err;
  PSAP_ADDR *psap = (PSAP_ADDR *)NULL;
  psap = get_cbp(node->name);
  /* set the Fox variable based on the datatype passed */
  switch(type){
  case FLOAT:
    myval.Float = (float)atof(node->value);
    status = FLOAT;
    err = om_set_confirm(node->name,VARIABLE,0,&myval.Float,&status,sizeof(myval.Float),psap);
    break;

  case INTEGER:
    myval.Integer = atoi(node->value);
    status = INTEGER;
    err = om_set_confirm(node->name,VARIABLE,0,&myval.Integer,&status,sizeof(myval.Integer),psap);
    break;

  case STRING:
    status = STRING;
    err = om_set_confirm(node->name,VARIABLE,0,node->value,&status,strlen(node->value),psap);
    break;

  case OM_BOOL:
    if (atoi(node->value)) myval.Om_bool = 1;
    else myval.Om_bool = 0;
    status = OM_BOOL;
    err = om_set_confirm(node->name,VARIABLE,0,&myval.Om_bool,&status,sizeof(myval.Om_bool),psap);
    break;

  case OM_LNG_INT:
  
    break;
  case OM_S_PKBOL:
 
    break;
  case OM_L_PKBOL:
  
    break;
  default:

    break;
  }

    if ( err != OM_SUCCESS ){
      fprintf(stderr,"Om write failed %s (return code %d)\n",node->name,err );
    }
  return(err);
}

/* function to return the fox pointer to the C:B.P requested */

PSAP_ADDR *get_cbp(char *str){ 
  struct cbpmem *mycbp;

  PSAP_ADDR temp_psap;


 /*  printf("entered get_cbp %s semid %d\n",str,sync_sem); */
  for(mycbp = shobj;mycbp < (shobj+maxshm);mycbp++){
    if(mycbp->name[0] != '\0'){
      if ( ! strcmp(mycbp->name,str)) { /* matches cbp */
	return(&mycbp->psap_ptr);
      }
    } else { /* no more in list */
      if ((global_find(str,VARIABLE,&temp_psap)) == OM_SUCCESS){
	update_db(&temp_psap,str); /* insert new entry in database */
	return(&mycbp->psap_ptr);
      } else {
	return((PSAP_ADDR *)NULL);
      }
      
    }

  }
  return((PSAP_ADDR *)NULL);
}


/* function to update shared memory database with new entry */
/* uses a semaphore to wait for any database entry changes  */
int update_db (PSAP_ADDR *tmp_psap,char *str) {
  struct sembuf sops;
  struct cbpmem *mycbp;
  /* initialise semaphore structure */
  sops.sem_num = 0;
  sops.sem_flg = 0;
  /* check if sem set -blocks until sem clears*/
  sops.sem_op = 0;
  semop(sync_sem,&sops,1); /* block until sem cleared */
  sops.sem_op = 1;
  semop(sync_sem,&sops,1); /* set semaphor to prevent another process */
  for(mycbp = shobj;mycbp < (shobj+maxshm);mycbp++){
    if(mycbp->name[0] == '\0'){
      memcpy(&mycbp->psap_ptr,tmp_psap,sizeof(PSAP_ADDR));
      strcpy(mycbp->name,str);
      sops.sem_op = -1;
      semop(sync_sem,&sops,1); /* clear semaphor to allow another process */
      return(0);
    }
  }
  sops.sem_op = -1;
  semop(sync_sem,&sops,1); /* clear semaphor to allow another process */
  return(1);
}
