/* RPC server for Fox om calls */
/* Refer to RPC -John Bloomer O'Reilly & Assoc Pg265 */
/* 
#---------------------------------------------------------------------+
#  Copyright (c) 2000 NRG Gladstone Operating Services.               |
#                All Rights Reserved.                                 |
#                                                                     |
#                                                                     |
#  This is free software; you can redistribute it and/or modify it    |
#  under the terms of the GNU General Public License as published by  |
#  the Free Software Foundation; either version 2 of the License, or  |
#  (at your option) any later version.                                |
#                                                                     |
#  This software is distributed in the hope that it will be useful,   |
#  but WITHOUT ANY WARRANTY; without even the implied warranty of     |
#  merchantability or fitness for a particular purpose. See the GNU   |
#  General Public License for more details.                           |
#                                                                     |
#  To obtain the GNU General Public License, write to:                |
#                                                                     |
#      Free Software Foundation, Inc.                                 |
#      59 Temple Place - Suite 330                                    |
#      Boston, MA  02111-1307                                         |
#      USA                                                            |
#---------------------------------------------------------------------+
*/

/*
# $Log: omserver.c,v $
# Revision 1.1  2000/03/19 22:27:25  dbond
# Initial revision
#
#
#
*/


/* ================= includes =============================== */
#include <stdio.h>
#include <string.h>


#include <rpc/rpc.h>
#include <signal.h>
#include <sys/wait.h>
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <fox/om_user.h>
#include <fox/om_ecode.h>
#include "omrd.h"
#include "ommem.h"
/* ================= defines ================================== */

#define MAXVALS 100
char omserver_v[] = "$Id: omserver.c,v 1.1 2000/03/19 22:27:25 dbond Exp $";
char omrd_x_vv[] = OMRD_X;

/* ================= prototypes =============================== */

void sigchild();
static void dispatch();
extern bool_t xdr_dir();

struct timeval tval;
static int count = 0;
int maxshm;

/* ================= code ===================================== */


main(int argc,char *argv[])
{
  SVCXPRT *transp;
  key_t key;
  int i;
  (void) pmap_unset(OMPROG,OMVERS);

  /* added ability to select number of shared memory slots */
  /* D Bond 9/9/97 */
  if ( argc == 2 ) {
    maxshm = atoi(argv[1]);
  } else {
    maxshm = MAXSHM;
  }

  transp = (SVCXPRT *)svcudp_create(RPC_ANYSOCK);
  if (transp == NULL) {
    (void) fprintf(stderr,"cannot create udp service. \n");
    exit(1);
  }

  if(!svc_register(transp,OMPROG,OMVERS,dispatch,IPPROTO_UDP)){
    (void)fprintf(stderr,"unable to register (DIRPROG,DIRVERS,udp.\n");
  exit(1);
  }
  tval.tv_sec = tval.tv_usec = 0;
  /* create shared memory area */
  key = 9876;
  if((i=shmget(key,sizeof(struct cbpmem)*maxshm,IPC_CREAT|0666)) <0) {
    fprintf(stderr,"shmget failed\n");
    exit(1);
  }
  if (((char *)shobj = shmat(i,NULL,0)) == (char *) -1){
       fprintf(stderr,"shmget failed\n");
  }
  memset((void *)shobj,'\0',sizeof(struct cbpmem)*maxshm);
  if((sync_sem=semget(key,1,IPC_CREAT|0666)) <0){
    fprintf(stderr,"semget failed\n");
    exit(1);
  }
/*   printf("set up mem\n"); */
  svc_run();
  (void)fprintf(stderr,"svc_run returned\n");
  exit(1);
}


static void
dispatch(rqstp,transp)
struct svc_req *rqstp;
SVCXPRT  *transp;
{
  enum clnt_stat clnt_stat;
  CLIENT *client;
  static omrd_res  result;
  namenode *node,*argp;
  int wr_result=0,type=INTEGER,first,status,chld;

  /* spawn a child process to service the request */
  signal(SIGCHLD,sigchild);
  count++;
  if((chld = fork())) {
    if (chld == -1){
      fprintf(stderr,"Child process fork failed\n");
    }
    return; /* Parent process*/
  }
  /* Child Process */

  /* rpc call ? */
  switch (rqstp->rq_proc){
  case NULLPROC:
    (void) svc_sendreply(transp,(xdrproc_t)xdr_void,0);
    return;
  case OMRD: /* omget request */
     argp=(namenode *)calloc(MAXVALS,sizeof(namenode));   
     if(!svc_getargs(transp,(xdrproc_t)xdr_namenode,(char *)argp)){  
      svcerr_decode(transp);
      return;
    }
  

    result.omrd_res_u.list = argp; /* initialise return list to array */

    /* loop through array servicing cbp omgets */
    for(node = (namenode *)argp ; node != NULL;node = node->pNext){ 
      omget(node);
/*       printf("%s  %s\n",node->name,node->value); */
    } 

    
    /* Send reply back no need for cleanup  */
    
    if(!svc_sendreply(transp,(xdrproc_t)xdr_omrd_res,(char *)&result)){
      svcerr_systemerr(transp);
      _exit(-1);
    }
    xdr_free((xdrproc_t)xdr_omrd_res,(char *)&result);
    free(argp); /* free list */
     _exit(1); 

/**************** OMWRITE Service **********************/
  case OMWR:

    argp=(namenode *)calloc(MAXVALS,sizeof(namenode));

    if(!svc_getargs(transp,(xdrproc_t)xdr_namenode,(char *)argp)){  
      svcerr_decode(transp);
      return;
    }
    /* loop through client list and get the data */
    for(node = argp ; node != NULL;node = node->pNext){

      for(first=0;node->name[first] != '\0';first++) 
	node->name[first] = toupper(node->name[first]);

      if (node->name[0] == '+'){
	if (!strcmp((char *)node->name,"+INT")){
	  type = INTEGER;
	} else if (!strcmp((char *)node->name,"+FLOAT")){
	  type = FLOAT;
	} else if (!strcmp((char *)node->name,"+STRING")){
	  type = STRING;
	} else if (!strcmp((char *)node->name,"+PBOOL")){
	  type = OM_S_PKBOL;
	} else if (!strcmp((char *)node->name,"+BOOL")){
	  type = OM_BOOL;
	}
      } else wr_result = omset(node,type);

    }
    if(!svc_sendreply(transp,(xdrproc_t)xdr_int,(char *)&wr_result)){
      svcerr_systemerr(transp);
      _exit(-1);
    }
    xdr_free((xdrproc_t)xdr_int,(char *)&wr_result);
    free(argp); /* free list */
    _exit(0);
    break;
  default:
    svcerr_noproc(transp);
    return;
  }
}



void sigchild() {
  int status;
  int DeadPid;
  while((DeadPid = wait(&status)) == -1);
  return;
}

